const mailService = require('pubsweet-component-mail-service')
const logger = require('@pubsweet/logger')
const helpers = require('./helpers')

module.exports = {
  setupAssignEmail: async (req, email, res, role) => {
    const url = `${req.protocol}://${req.get('host')}`
    let emailType
    switch (role) {
      case 'handlingEditor':
        emailType = 'assign-handling-editor'
        break
      case 'author':
        emailType = 'add-author'
        break
      default:
        return res
          .status(400)
          .json({ error: `Role ${role} cannot be used with an assigned email` })
    }
    try {
      await mailService.setupAssignEmail(email, emailType, url)
      return res.status(200).json({})
    } catch (e) {
      logger.error(e)
      return res.status(500).json({ error: 'Email could not be sent.' })
    }
  },
  setupNewUserEmail: async (req, res, email, role, UserModel) => {
    let user
    try {
      user = await UserModel.findByEmail(email)
    } catch (e) {
      const notFoundError = await helpers.handleNotFoundError(e, 'user')
      return res.status(notFoundError.status).json({
        error: notFoundError.message,
      })
    }
    if (user.passwordResetToken === undefined) {
      return res
        .status(400)
        .json({ error: 'User does not have a password reset token' })
    }
    const url = `${req.protocol}://${req.get('host')}`
    let emailType
    switch (role) {
      case 'handlingEditor':
      case 'editorInChief':
      case 'admin':
        emailType = 'invite'
        break
      case 'author':
        emailType = 'invite-author'
        break
      default:
        return res.status(400).json({
          error: `Role ${role} cannot be used with a password reset email`,
        })
    }
    try {
      await mailService.setupInviteEmail(
        email,
        emailType,
        user.passwordResetToken,
        url,
      )
      return res.status(200).json({})
    } catch (e) {
      logger.error(e)
      return res.status(500).json({ error: 'Email could not be sent.' })
    }
  },
}
