import { get } from 'lodash'
import request, { create } from 'pubsweet-client/src/helpers/api'
import { withJournal } from 'xpub-journal'
import { loginUser } from 'pubsweet-component-login/actions'
import { SubmissionError } from 'redux-form'
import {
  compose,
  withState,
  withProps,
  withHandlers,
  lifecycle,
} from 'recompose'

import SignUpInvitation from './SignUpInvitationForm'

const login = (dispatch, values, history) =>
  dispatch(loginUser(values))
    .then(() => {
      history.push('/')
    })
    .catch(error => {
      const err = get(error, 'response')
      if (err) {
        const errorMessage = get(JSON.parse(err), 'error')
        throw new SubmissionError({
          password: errorMessage || 'Something went wrong',
        })
      }
    })

const confirmUser = (email, token, history) => (values, dispatch) => {
  const request = { ...values, email, token }
  if (values) {
    return create('/users/reset-password', request)
      .then(r => {
        const { username } = r
        const { password } = values
        login(dispatch, { username, password }, history)
      })
      .catch(error => {
        const err = get(error, 'response')
        if (err) {
          const errorMessage = get(JSON.parse(err), 'error')
          throw new SubmissionError({
            _error: errorMessage || 'Something went wrong',
          })
        }
      })
  }
}

export default compose(
  withJournal,
  withState('step', 'changeStep', 0),
  withState('initialValues', 'setInitialValues', {}),
  withProps(({ location }) => {
    const params = new URLSearchParams(location.search)
    const email = params.get('email')
    const token = params.get('token')
    return { email, token }
  }),
  withHandlers({
    nextStep: ({ changeStep }) => () => changeStep(step => step + 1),
    prevStep: ({ changeStep }) => () => changeStep(step => step - 1),
    submitConfirmation: ({ email, token, history }) =>
      confirmUser(email, token, history),
  }),
  lifecycle({
    componentDidMount() {
      const { setInitialValues, token } = this.props
      const encodedUri = `?passwordResetToken=${token}`
      request(`/users/${encodedUri}`)
        .then(res => {
          setInitialValues(v => res.users[0])
        })
        .catch(err => {
          this.setState({ error: err.response })
        })
    },
  }),
)(SignUpInvitation)
