import { get } from 'lodash'
import { get as apiGet, create, remove } from 'pubsweet-client/src/helpers/api'

const GET_REVIEWERS_REQUEST = 'GET_REVIEWERS_REQUEST'
const GET_REVIEWERS_ERROR = 'GET_REVIEWERS_ERROR'
const GET_REVIEWERS_SUCCESS = 'GET_REVIEWERS_SUCCESS'

export const getReviewersRequest = () => ({
  type: GET_REVIEWERS_REQUEST,
})

export const getReviewersError = error => ({
  type: GET_REVIEWERS_ERROR,
  error,
})

export const getReviewersSuccess = reviewers => ({
  type: GET_REVIEWERS_SUCCESS,
  payload: { reviewers },
})

const INVITE_REVIEWER_REQUEST = 'INVITE_REVIEWER_REQUEST'
const INVITE_REVIEWER_SUCCESS = 'INVITE_REVIEWER_SUCCESS'
const INVITE_REVIEWER_ERROR = 'INVITE_REVIEWER_ERROR'

const inviteRequest = () => ({
  type: INVITE_REVIEWER_REQUEST,
})

const inviteSuccess = () => ({
  type: INVITE_REVIEWER_SUCCESS,
})

const inviteError = error => ({
  type: INVITE_REVIEWER_ERROR,
  error,
})

const initialState = {
  fetching: {
    invite: false,
    reviewers: false,
  },
  error: null,
  reviewers: [],
}

// selectors
export const selectReviewers = state => get(state, 'reviewers.reviewers') || []
export const selectFetchingReviewers = state =>
  get(state, 'reviewers.fetching.reviewers') || false
export const selectFechingInvite = state =>
  get(state, 'reviewers.fetching.invite') || false

export const getCollectionReviewers = collectionId => dispatch => {
  dispatch(getReviewersRequest())
  return apiGet(`/collections/${collectionId}/invitations?role=reviewer`).then(
    r => dispatch(getReviewersSuccess(r)),
    err => dispatch(getReviewersError(err)),
  )
}

export const inviteReviewer = (reviewerData, collectionId) => dispatch => {
  dispatch(inviteRequest())
  return create(`/collections/${collectionId}/invitations`, {
    ...reviewerData,
    role: 'reviewer',
  }).then(() => dispatch(inviteSuccess()), err => dispatch(inviteError(err)))
}

export const revokeReviewer = (invitationId, collectionId) => dispatch => {
  dispatch(inviteRequest())
  return remove(
    `/collections/${collectionId}/invitations/${invitationId}`,
  ).then(() => dispatch(inviteSuccess()), err => dispatch(inviteError(err)))
}

export default (state = initialState, action = {}) => {
  switch (action.type) {
    case GET_REVIEWERS_REQUEST:
      return {
        ...state,
        fetching: {
          ...state.fetching,
          reviewers: true,
        },
      }
    case GET_REVIEWERS_ERROR:
      return {
        ...state,
        fetching: {
          ...state.fetching,
          reviewers: false,
        },
        error: action.error,
      }
    case GET_REVIEWERS_SUCCESS:
      return {
        ...state,
        fetching: {
          ...state.fetching,
          reviewers: false,
        },
        reviewers: action.payload.reviewers,
      }
    case INVITE_REVIEWER_REQUEST:
      return {
        ...state,
        fetching: {
          ...state.fetching,
          invite: true,
        },
      }
    case INVITE_REVIEWER_SUCCESS:
      return {
        ...state,
        fetching: {
          ...state.fetching,
          invite: false,
        },
      }
    case INVITE_REVIEWER_ERROR:
      return {
        ...state,
        fetching: {
          ...state.fetching,
          invite: false,
        },
        error: action.error,
      }
    default:
      return state
  }
}
